﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Prsi
{
    /// <summary>
    /// This is type of rule for each turn.
    /// </summary>
    public enum RULE
    {
        ESO,
        STANDARD,
        SEVEN,
        USED
    }

    /// <summary>
    /// This is main logic and rules of game.
    /// </summary>
    public class Rules : IRules
    {
        private const int COUNT_OF_CARDS_ON_START = 4;
        private const string VALUE_OF_SEVEN = "7";
        private const string VALUE_OF_ESO = "A";
        private const string VALUE_OF_EMPTY_CARD = " ";
        private const string VALUE_OF_CHANGING_CARD = "Q";
        private const int PENALTY_FOR_SEVEN = 2;

        public int PenaltyForSeven { get { return PENALTY_FOR_SEVEN; } }

        private RULE lastRule;
        public RULE LastRule { get { return lastRule; } set { lastRule = value; } }

        private int countOfSevens;
        public int CountOfSevens { get { return countOfSevens; } set { countOfSevens = value; } }

        private List<Player> players;
        private PlayedCards playedCards;
        private Deck deck;
        private Printer printer;

        public Rules(List<Player> players, PlayedCards playedCards, Deck deck)
        {
            this.players = players;
            this.playedCards = playedCards;
            this.deck = deck;

            printer = new Printer(players, ConsoleColor.Yellow, ConsoleColor.Gray);
            countOfSevens = 0;
        }

        public RULE GetRuleNumber(Card lastPlayedCard)
        {
            if (lastRule == RULE.USED)
                return RULE.STANDARD;

            switch (lastPlayedCard.Value)
            {
                case VALUE_OF_ESO:
                    lastRule = RULE.ESO;
                    return lastRule;

                case VALUE_OF_SEVEN:
                    lastRule = RULE.SEVEN;
                    return lastRule;

                default:
                    lastRule = RULE.STANDARD;
                    return lastRule;
            }
        }

        public List<int> FindValideCards(Card lastPlayedCard, List<Card> playersHand)
        {
            if (lastRule == RULE.USED)
                return RuleForStandardCard(lastPlayedCard.Value, lastPlayedCard.Color, playersHand);

            switch (lastPlayedCard.Value)
            {
                case VALUE_OF_ESO:
                    return RuleForSpecialCardsValue(lastPlayedCard.Value, playersHand);

                case VALUE_OF_SEVEN:
                    return RuleForSpecialCardsValue(lastPlayedCard.Value, playersHand);

                case VALUE_OF_EMPTY_CARD:
                    return RuleForSpecialCardsColor(lastPlayedCard.Color, playersHand);

                default:
                    return RuleForStandardCard(lastPlayedCard.Value, lastPlayedCard.Color, playersHand);
            }
        }

        private List<int> RuleForSpecialCardsValue(string value, List<Card> playersHand)
        {
            List<int> valideCardIndexes = new List<int>();
            foreach (Card card in playersHand)
            {
                if (card.Value == value)
                    valideCardIndexes.Add(playersHand.IndexOf(card));
            }
            return valideCardIndexes;
        }

        private List<int> RuleForSpecialCardsColor(COLOR color, List<Card> playersHand)
        {
            List<int> valideCardIndexes = new List<int>();
            foreach (Card card in playersHand)
            {
                if (card.Color == color || card.Value == VALUE_OF_CHANGING_CARD)
                    valideCardIndexes.Add(playersHand.IndexOf(card));
            }
            return valideCardIndexes;
        }

        private List<int> RuleForStandardCard(string value, COLOR color, List<Card> playersHand)
        {
            List<int> valideCardIndexes = new List<int>();
            foreach (Card card in playersHand)
            {
                if (card.Value == value || card.Color == color || card.Value == VALUE_OF_CHANGING_CARD)
                    valideCardIndexes.Add(playersHand.IndexOf(card));
            }
            return valideCardIndexes;
        }

        public void DealCards()
        {
            for (int i = 0; i < COUNT_OF_CARDS_ON_START; i++)
            {
                for (int j = 0; j < players.Count(); j++)
                {
                    DrawCard(j);
                }
            }
        }

        public void PutCartOnTable(Card card)
        {
            playedCards.AddCard(card);
            if (card.Value == VALUE_OF_SEVEN)
                countOfSevens++;
        }

        public void ProcessAnswer(int answer, RULE rule, int playerWhoPlay, List<int> possibilities)
        {
            if (answer == 0)
            {
                switch (rule)
                {
                    case RULE.SEVEN:
                        for (int i = 0; i < countOfSevens * PENALTY_FOR_SEVEN; i++)
                            if (!DrawCard(playerWhoPlay))
                                break;

                        countOfSevens = 0;
                        lastRule = RULE.USED;
                        break;

                    case RULE.STANDARD:
                        DrawCard(playerWhoPlay);
                        break;

                    case RULE.ESO:
                        lastRule = RULE.USED;
                        break;

                }
            }
            else
            {
                lastRule = rule;
                int cardIndex = possibilities[answer - 1];
                Card playedCard = players[playerWhoPlay].Hand[cardIndex];

                players[playerWhoPlay].Play(playedCard);
                playedCards.AddCard(playedCard);

                if (playedCard.Value == VALUE_OF_CHANGING_CARD)
                {
                    printer.PrintPossibleColorsToChange();
                    int newColor = Questions.GetInstance().AnswerInRange("Tvá volba [0-3]: ", 0, 3);
                    playedCards.AddCard(new Card(VALUE_OF_EMPTY_CARD, (COLOR)newColor + 3));
                }

                if (playedCard.Value == VALUE_OF_SEVEN)
                    countOfSevens++;
            }
        }

        private bool DrawCard(int index)
        {
            if (deck.IsEmpty())
            {
                if (playedCards.Cards.Count() >= 1)
                {
                    printer.PrintError("Nastala chyba: v balíčku, ani v odkládacím balíčku nejsou žádné další karty a\nproto si nemůžeš líznout.");
                    return false;
                }
                deck.AddCards(playedCards.UsePlayedCardsLikeNewDeck());
            }

            players[index].AddCard(deck.GetTopCardFromDeck());
            return true;
        }

        public bool FillEmptyDeck()
        {
            if (deck.IsEmpty() && playedCards.Cards.Count() > 1)
            {
                deck.AddCards(playedCards.UsePlayedCardsLikeNewDeck());
                return true;
            }
            return false;
        }

        public int FindMinNumberOfPossibilities(int possibilitiesCount, int playerWhoPlay)
        {
            if (deck.IsEmpty() && playedCards.Cards.Count() == 1)
                if (possibilitiesCount > 0)
                    return 1;
                else
                {
                    printer.PrintError("Hráč " + players[playerWhoPlay].Name + " byl přeskočen, protože nemůže hrát.");
                    return -1;
                }
            else
            {
                return 0;
            }
        }
    }
}
